import os, multiprocessing
from gensim.utils import simple_preprocess
from gensim.models import Word2Vec
from .utils import remove_stopwords_id

REAL_PATH = os.path.dirname(os.path.realpath(__file__))
FILE_DIR = REAL_PATH + "/simple_preprocess/sentences.txt"
MODEL_DIR = REAL_PATH + "/model/fin.model"
SIZE = 100
WINDOW = 5
ITER = 10


def create_corpus():
    sentences = list(
        map(remove_stopwords_id, open(FILE_DIR, encoding="utf-8").read().splitlines())
    )
    corpus = list(map(simple_preprocess, sentences))
    print(corpus[:2], "\nSentences: -->", len(corpus))
    uniqset = set(word for l in corpus for word in l)
    print(len(uniqset), "Unique Terms")
    return corpus


def create_word2vec(save=False):
    corpus = create_corpus()
    model = Word2Vec(
        corpus,
        size=SIZE,
        window=WINDOW,
        min_count=1,
        workers=multiprocessing.cpu_count(),
        iter=ITER,
    )
    if save:
        model.save(MODEL_DIR)
        print("Model Saved:", MODEL_DIR)

    return model


# print(model.wv.most_similar("kartu"))
# [('kredit', 0.5386360287666321),
# ('berbagai', 0.5347478985786438),
# ('sejalan', 0.502410888671875),
# ('tahapan', 0.49144789576530457),
# ('menerapkan', 0.41903647780418396),
# ('terkemuka', 0.3761982023715973),
# ('tahun', 0.3546695113182068),
# ('atm', 0.3403625190258026),
# ('nama', 0.2962668836116791)]


if __name__ == "__main__":
    model = create_word2vec()

print(model.wv.most_similar("bca"))

# len(model.wv.vocab) -> 1158
# model.corpus_total_words -> 3320
# model.wv['bca'] # ndarray of length 10
# model.wv.vocab['atm'].count -> 5



from gensim.utils import simple_preprocess

sentence = "BCA mengajukan permohonan kepada Bank Indonesia agar diperbolehkan mengeluarkan dan mengedarkan kartu kredit atas nama BCA yang berlaku internasional. Untuk itu, BCA bekerjasama dengan MasterCard. BCA juga memperluas jaringan kantor cabang secara agresif sejalan dengan deregulasi sektor perbankan di Indonesia. BCA mengembangkan berbagai produk dan layanan maupun pengembangan teknologi informasi, dengan menerapkan online system untuk jaringan kantor cabang, dan meluncurkan Tabungan Hari Depan (Tahapan) BCA. Pada tahun 1990-an BCA mengembangkan alternatif jaringan layanan melalui ATM (Anjungan Tunai Mandiri atau Automated Teller Machine). Pada tahun 1991, BCA mulai menempatkan 50 unit ATM di berbagai tempat di Jakarta. Pengembangan jaringan dan fitur ATM dilakukan secara intensif. BCA bekerja sama dengan institusi terkemuka, antara lain PT Telkom untuk pembayaran tagihan telepon melalui ATM BCA. BCA juga bekerja sama dengan Citibank agar nasabah BCA pemegang kartu kredit Citibank dapat melakukan pembayaran tagihan melalui ATM BCA."
corpus = [word.lower() for word in sentence.split()]
print(f'{corpus}, total word: {len(corpus)}')
print(' --- \n')
corpus2 = simple_preprocess(sentence)
print(f'{corpus2}, total word: {len(corpus2)}')
print(' --- \n')
print([word for word in corpus if word not in corpus2])
    


from unittest import TestCase
from elang.word2vec.utils import (
    remove_region_id,
    remove_stopwords_id,
    remove_vulgarity_id,
)


class TestRemove(TestCase):
    def test_remove_region(self):
        sen = "Sudah pernah naik kereta dari Jakarta ke Bandung dari stasiun nggak?"
        res = remove_region_id(sen)
        self.assertLess(len(res.split(" ")), len(sen.split(" ")))



import os
from gensim.utils import simple_preprocess
from gensim.models import Word2Vec
from utils import remove_stopwords_id

SIZE = 2
REAL_PATH = os.path.dirname(os.path.realpath(__file__))
FILE_DIR = REAL_PATH + "/simple_preprocess/demosentences.txt"
MODEL_DIR = REAL_PATH + f"/model/demo{SIZE}d.model"
WINDOW = 2
ITER = 1000
WORKERS = 4


def create_corpus():
    sentences = list(map(remove_stopwords_id, open(FILE_DIR).read().splitlines()))
    corpus = list(map(simple_preprocess, sentences))
    print(corpus[7])
    print("Sentences: -->", len(corpus))
    uniqset = set(word for l in corpus for word in l)
    print("Unique Terms -->", len(uniqset))
    print(uniqset)
    return corpus


def hash(astring):
    return ord(astring[0])


def create_word2vec(save=False):
    corpus = create_corpus()
    model = Word2Vec(
        corpus,
        size=SIZE,
        window=WINDOW,
        min_count=1,
        workers=WORKERS,
        iter=ITER,
        seed=3,
        hashfxn=hash,
    )
    if save:
        model.save(MODEL_DIR)
        print("Model Saved:", MODEL_DIR)

    return model


if __name__ == "__main__":
    model = create_word2vec(save=False)


# Try: kartu, rekening
print(model.wv.most_similar("bca"), "\n", "----")
print(model.wv.doesnt_match("mandiri agustus uob bca".split()), "\n", "----")
print(model.wv.doesnt_match("kartu airbnb kredit debit".split()), "\n", "----")
print(
    model.wv.doesnt_match("perusahaan sekretaris karyawan dekade".split()), "\n", "----"
)




import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm


def plot2d(model, words=None, method="TSNE", targets=[], *args, **kwargs):
    """plot2d Plot word embeddings in 2-dimension
    
    Create a Matplotlib plot to display word embeddings in 2 dimensions, using a specified dimensionality reduction method (`method`) if the word vectors have more than 2 dimensions.
    Optionally accepts a `list` for the `words` parameter, to display only a subset of words from the `model`'s dictionary. When `None`, it will use the first 500 words of the model.  
    Optionally accepts a `list` for the `targets` parameter, to emphasize in bold fontface a subset of words
    
    Any other parameters specified using `*args` or `**kwargs` is unpacked and passed on to the underlying dimensionality reduction method in `sklearn`.

    :param model: An instance of Word2Vec
    :type model: Word2Vec
    :param words: List of words to render in plot -- when None all words in the `model` are plotted, defaults to None
    :type words: list or None, optional
    :param method: Method for dimensionality reduction, defaults to "TSNE"
    :type method: str, optional
    :param targets: List of words to be emphasized using a bold font in the plot, defaults to []
    :type targets: list, optional
    :return: A matplotlib figure
    :raises AssertionError: Ensure `model` is size 2 (2-dimension word vectors) or higher
    """
    assert (
        model.vector_size >= 2
    ), "This function expects a model of size 2 (2-dimension word vectors) or higher."

    if isinstance(targets, str):
        try:
            targets = [targets]
        except TypeError:
            raise TypeError("The targets parameter expect a python list")

    if words is None:
        words = [word for word in list(model.wv.vocab)[1:500]]
    else:
        try:
            words = [word for word in words if word in model.wv.vocab]
        except TypeError as e:
            raise TypeError("The 'words' parameter expect a python list")

    word_vec = np.array([model.wv[word] for word in words])
    try:
        if model.vector_size > 2:
            if method == "PCA":
                from sklearn.decomposition import PCA
                word_vec = PCA(2, *args, **kwargs).fit_transform(word_vec)

            elif method == "TSNE":
                from sklearn.manifold import TSNE
                word_vec = TSNE(2, *args, **kwargs).fit_transform(word_vec)

            else:
                raise AssertionError(
                    "Model must be one of PCA or TSNE for model with greater than 2 dimensions"
                )

        with plt.style.context("seaborn-pastel"):
            plt.figure(figsize=(7, 5), dpi=180)
            plt.scatter(
                word_vec[:, 0], word_vec[:, 1], s=5, alpha=0.3, edgecolors="k", c="c"
            )

            for word, (x, y) in zip(words, word_vec):
                if word in [elem.lower() for elem in targets if targets]:
                    plt.text(x - 0.02, y + 0.02, word, fontsize=5, weight="bold")
                else:
                    plt.text(x - 0.02, y + 0.02, word, fontsize=5, alpha=0.5)

        plt.show()
    except ValueError as e:
        raise ValueError("Fail to perform dimensionality reduction.")

def plotNeighbours(model, words, k=10, method="TSNE", draggable=False, *args, **kwargs):
    """plotNeighbours Plot and color the `k` nearest neighbors for each word in 2-dimension 

    Create a Matplotlib plot to display word embeddings and their k-nearest neighbors in 2 dimensions, using a specified dimensionality reduction method (`method`) if the word vectors have more than 2 dimensions.
    Set`draggable` to `True` for a draggable legend in the resulting plot. 
    
    Any other parameters specified using `*args` or `**kwargs` is unpacked and passed on to the underlying dimensionality reduction method in `sklearn`.

    :param model: An instance of Word2Vec
    :type model: Word2Vec
    :param words: List of centroid words to render in plot
    :type words: List
    :param k: Number of neighbors for each word, defaults to 10
    :type k: int, optional
    :param method: Method for dimensionality reduction, defaults to "TSNE"
    :type method: str, optional
    :param draggable: Set to `True` if a draggable legend box is preferred, defaults to False
    :type draggable: bool, optional
    :raises AssertionError: Ensure `model` is size 2 (2-dimension word vectors) or higher
    """
    assert (
        model.vector_size >= 2
    ), "This function expects a model of size 2 (2-dimension word vectors) or higher."

    if isinstance(words, str):
        try:
            targets = [targets]
        except TypeError:
            raise TypeError("The targets parameter expect a python list")

    embedding_clusters = []
    word_clusters = [] # (7,10)
    
    try:
        words = [word for word in words if word in model.wv.vocab]
    except TypeError as e:
            raise TypeError("The 'words' parameter expect a python list")

    for word in words:
        neighbors = [] 
        embeddings = []
        for similar, _ in model.wv.most_similar(word, topn=k):
            neighbors.append(similar)
            embeddings.append(model.wv[similar])
        embedding_clusters.append(embeddings)
        word_clusters.append(neighbors)
    embedding_clusters = np.array(embedding_clusters)  
    cent_n, neigh_n, dim_n = embedding_clusters.shape # 7, 10, 50 (centroid, neighborhood, dimensions)
    embedding_clusters = embedding_clusters.reshape(cent_n * neigh_n, dim_n) # (70, 50)
    
    if model.vector_size > 2:
        if method == "PCA":
            from sklearn.decomposition import PCA
            word_vec = PCA(2, *args, **kwargs).fit_transform(embedding_clusters)

        elif method == "TSNE":
            from sklearn.manifold import TSNE
            word_vec = TSNE(2, *args, **kwargs).fit_transform(embedding_clusters)

        else:
            raise AssertionError(
                "Model must be one of PCA or TSNE for model with greater than 2 dimensions"
            )
    else:
        word_vec = embedding_clusters

    word_vec = word_vec.reshape(cent_n, neigh_n, -1) # (7,10,2)
    with plt.style.context("seaborn-pastel"):
        plt.rc('legend', fontsize=7, fancybox=True, framealpha=0.8, facecolor="#777777", edgecolor="#000000")
        plt.rc('font', size=7)
        plt.figure(figsize=(7, 5), dpi=180)
        cmx = cm.get_cmap('Pastel1')
        colors = cmx(np.linspace(0,1,len(words))) # (7,4)
        for word, embedding, neighbor, color in zip(words, word_vec, word_clusters, colors):
            x = embedding[:,0]
            y = embedding[:,1]
            plt.scatter(x, y, color=color, alpha=1, label=word)

            for i, word in enumerate(neighbor):
                plt.annotate(word, alpha=0.6, xy=(x[i], y[i]), size=5)

        if draggable:
            leg = plt.legend()
            leg.set_draggable(state=True)
        else:
            leg = plt.legend(loc="lower left", ncol=min(5, len(words)))
            plt.setp(leg.get_texts(), color='w')
            

    plt.show()



from unittest import TestCase
from elang.word2vec.builder import (
    build_from_wikipedia,
    build_from_wikipedia_branch,
    build_from_wikipedia_random,
)


class TestBuildRandomly(TestCase):
    def test_build_random(self):
        pass



import os
import re

realpath = os.path.dirname(os.path.realpath(__file__))

def _open_file(filename):
    filepath = realpath + "/negative/" + filename
    wordlist = list(open(filepath).read().splitlines())
    return wordlist


def _remove_words(sentence, words2remove):
    for word in words2remove:
        sentence = re.sub(r"\b" + word + r"\b", "", sentence.lower())
        sentence = re.sub(r"\s+", " ", sentence).strip()
    return sentence


# main cleansing function
def remove_stopwords_id(sentence):
    """remove_stopwords_id Removes Bahasa Indonesia Stopwords
 
    Stopwords are generally the most common "function" words in a language, and they're routinely eliminated in natural language processing tasks.
    Bahasa Indonesia stopwords (eg. "ya", "kan", "dong", "loh") are removed from the input string.
    
    :param sentence: An input string
    :type sentence: str
    :return: A string where common Bahasa Indonesia stopwords are filtered out 
    :rtype: str
    """
    stopwords = _open_file("stopwords-id.txt")
    sentence = _remove_words(sentence, stopwords)
    return sentence


def remove_region_id(sentence):
    """remove_region_id Removes name of places in Indonesia 
    Regions are name of places (provinces and cities) in Indonesia (eg. "Jakarta", "Bali", "Sukabumi")
    
    :param sentence: An input string
    :type sentence: str
    :return: A string where common name of places in Indonesia are filtered out 
    :rtype: str
    """
    regions = _open_file("indonesian-region.txt")
    sentence = _remove_words(sentence, regions)
    return sentence


def remove_vulgarity_id(sentence):
    """remove_vulgarity_id Removes uncivilised words in Bahasa Indonesia 
    Prevent words such as "anjir", "babi" etc to be included in natual language generation tasks
    
    :param sentence: An input string
    :type sentence: str
    :return: A string where common swear words in Indonesia are filtered out 
    :rtype: str
    """
    swears = _open_file("swear-words.txt")
    sentence = _remove_words(sentence, swears)
    return sentence

def remove_calendar_id(sentence):
    """remove_datetime_id Removes common "calendar words" in Bahasa Indonesia 
    Calendar words include day of weeks (eg. senin, selasa, ...), months (`maret`, 'juni'), and their abbreviated forms (`okt`, 'jul') 

    :param sentence: An input string
    :type sentence: str
    :return: A string where calendar words in Indonesia are filtered out 
    :rtype: str
    """
    swears = _open_file("calendar-words.txt")
    sentence = _remove_words(sentence, swears)
    return sentence



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

exec(open("elang/version.py").read())

setuptools.setup(
    name="elang",
    version=__version__,
    author="Samuel Chan, Tomy Tjandra",
    author_email="samuel@algorit.ma",
    description="Word Embedding(E) utilities for Language(Lang) Models",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/onlyphantom/elang",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "gensim",
        "scikit-learn",
        "matplotlib",
        "tqdm",
        "beautifulsoup4",
    ],
    keywords="nlp bahasa indonesia indonesian natural language corpus word2vec gensim embedding nltk",
    test_suite="nose.collector",
    tests_require=["nose"],
)
